% /***************************************************************************
% *
% * Copyright 2002 by the Massachusetts Institute  of Technology.   All
% * rights reserved.
% *
% * Developed  by the Affective Computing Section
% * at the Media Laboratory, MIT, Cambridge, Massachusetts.
% *
% * Permission to use, copy, or modify this software and  its documentation
% * for  educational  and  research purposes only and without fee  is hereby
% * granted, provided  that this copyright notice and the original authors's
% * names appear  on all copies and supporting documentation.  If individual
% * files are  separated from  this  distribution directory  structure, this
% * copyright notice must be included.  For any other uses of this software,
% * in original or  modified form, including but not limited to distribution
% * in whole or in  part, specific  prior permission  must be  obtained from
% * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
% * basis  of  a commercial  software  or  hardware product  without first
% * obtaining appropriate licenses  from MIT.  MIT. makes no representations
% * about the suitability of this  software for any purpose.  It is provided
% * "as is" without express or implied warranty.
% *
% * Author: Ashish Kapoor (kapoor@mit.edu)
% * Date  : May,17th 2002
% *
% **************************************************************************/

function [viterbi_path, viterbi_score] = viterbi(pi, a, b, num_states, obs)

%*****************************************************************
%Preprocessing
pi_tilda = log(pi);
b_tilda = log(b);
a_tilda = log(a);

T=size(obs,1);
delta_tilda = zeros(T,num_states);
psi = zeros(T,num_states);
%*****************************************************************
%Initialization
delta_tilda(1,:) = pi_tilda' + b_tilda(:,obs(1)+1)';
psi(1,:) = zeros(1,num_states);

%*****************************************************************
%Recursion
for t=2:T
    for j=1:num_states
        [max_val, max_index] = max(delta_tilda(t-1,:) + a_tilda(:,j)');
        delta_tilda(t,j) = max_val + b_tilda(j, obs(t)+1);
        psi(t,j) = max_index;
    end
end

%*****************************************************************
%Termination and Backtracking
[max_val, max_index] = max(delta_tilda(T,:));
viterbi_score = max_val;

viterbi_path=[max_index];

for t=1:T-1
    viterbi_path = [psi(T-t+1, viterbi_path(1)) viterbi_path];
end