% /***************************************************************************
% *
% * Copyright 2002 by the Massachusetts Institute  of Technology.   All
% * rights reserved.
% *
% * Developed  by the Affective Computing Section
% * at the Media Laboratory, MIT, Cambridge, Massachusetts.
% *
% * Permission to use, copy, or modify this software and  its documentation
% * for  educational  and  research purposes only and without fee  is hereby
% * granted, provided  that this copyright notice and the original authors's
% * names appear  on all copies and supporting documentation.  If individual
% * files are  separated from  this  distribution directory  structure, this
% * copyright notice must be included.  For any other uses of this software,
% * in original or  modified form, including but not limited to distribution
% * in whole or in  part, specific  prior permission  must be  obtained from
% * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
% * basis  of  a commercial  software  or  hardware product  without first
% * obtaining appropriate licenses  from MIT.  MIT. makes no representations
% * about the suitability of this  software for any purpose.  It is provided
% * "as is" without express or implied warranty.
% *
% * Author: Ashish Kapoor (kapoor@mit.edu)
% * Date  : May,17th 2002
% *
% **************************************************************************/

function pr = log_prob_obs_hmm(pi, a, b, num_states, obs)
%Function to compute probability of the observation given the parameters of HMM (P(O|Lambda))

%Memory Allocation
T = size(obs,1);
alpha_hat = zeros(T, num_states);
alpha_hat_hat = zeros(T, num_states);
c = zeros(T,1);

%**********************************************************************************
%Initialization

%Initializing alpha_hat_hat; 
%****AVOID FOR LOOPS****
%To DO:
%for i=1:num_states
%    alpha_hat_hat(1,i)=pi(i)*b(i,obs(1)+1);
%end
%Eqivalent MATLAB code:
alpha_hat_hat(1,:)=pi'.*b(:,obs(1)+1)';

c(1)=1/sum(alpha_hat_hat(1,:));

alpha_hat(1,:)=c(1)*alpha_hat_hat(1,:);

%**********************************************************************************
%Recursion
for t=2:T
    
    %Again avoiding for loop
    %for j=1:num_states
    %    alpha_hat_hat(t,j)=sum(alpha_hat(t-1,:).*a(:,j)'*b(j,obs(t)+1));
    %end
    %Equivalent line of code below
    alpha_hat_hat(t,:)=(alpha_hat(t-1,:)*a(:,:)).*b(:,obs(t)+1)';
    
    c(t)=1/sum(alpha_hat_hat(t,:));
    
    alpha_hat(t,:)=c(t)*alpha_hat_hat(t,:);
       
end

%**********************************************************************************
%Termination
pr=-sum(log(c));
